from leinwand import Leinwand
from moebel import Moebel
import math
class Badewanne(Moebel):
    """
    Stellt eine Badewanne im Möbelplaner dar.
    """
    def __init__(self,xPosition=20,yPosition=150,
                 breite=200,tiefe=80,winkel=0,
                 randfarbe='blau',fuellfarbe='transparent'):
        """
        Erzeugt eine Badewanne an der gegebenen Position.
        xPosition: die x-Koordinate der Mitte der Badewanne  
        yPosition: die y-Koordinate der Mitte der Badewanne
        randfarbe: die Randfarbe, in der die Badewanne angezeigt werden soll.
               Dies sollte einer der Farbnamen der Klasse Farben sein.  
        """
        super().__init__(xPosition,yPosition,breite,tiefe,winkel,randfarbe,fuellfarbe)

    def erzeugeFigur(self):
        """
        Erzeugt die Figur, die eine Badewanne auf der Leinwand darstellt.
        """
        if not self.gibFormen():
            x,y,b,t,lw = self.gibX(),self.gibY(),self.gibB(),self.gibT(),self.gibLW()
            formen = []
            rand = lw.polygon(x,y, x+b,y, x+b,y+t, x,y+t)
            formen.append(rand)
            innenabstand = t//10
            innenrechts = lw.linie(x+innenabstand,y+innenabstand,
                                   x+b-t/2-innenabstand,y+innenabstand)
            formen.append(innenrechts)
            innenlinks = lw.linie(x+innenabstand,y+t-innenabstand,
                                  x+b-t/2-innenabstand,y+t-innenabstand)
            formen.append(innenlinks)
            innenfuss = lw.linie(x+innenabstand,y+innenabstand,
                                 x+innenabstand,y+t-innenabstand)
            formen.append(innenfuss)
            # Drehzentrum:
            xm,ym=x+b/2,y+t/2
            xO,yO=x+b-t/2-innenabstand,y+innenabstand
            xU,yU=x+b-t/2-innenabstand,y+t-innenabstand
            xO,yO=self.drehung(xO,yO,xm,ym,self.gibW())
            xU,yU=self.drehung(xU,yU,xm,ym,self.gibW())
            kopfteil = lw.bogen(xO,yO,
                                xU,yU,
                                start=self.gibW(),extent=180)
            formen.append(kopfteil)
            abfluss = lw.ellipse(x+3*innenabstand,y+(t-innenabstand)/2,
                                 x+4*innenabstand,y++(t+innenabstand)/2)
            formen.append(abfluss)
            return formen

    def drehung(self,x,y,xm,ym,winkel):
        bogenmass=math.radians(winkel)
        xe=(x-xm)*math.cos(bogenmass)-(y-ym)*math.sin(bogenmass)+xm
        ye=(x-xm)*math.sin(bogenmass)+(y-ym)*math.cos(bogenmass)+ym
        return xe,ye
            
