# -*- coding: utf-8 -*-
# raumplanerModell.py
## die erste Variante (ohne Mausereignisse)

"""Eine eigene Modellklasse für die Moebel-Klassen
übernimmt die gesamte Verwaltung der erzeugten Möbel
einschließlich Speichern und Laden,
sowie Auswahl
Die Klasse ist als Singleton modelliert"""

import pickle # Persistenz realisieren

from stuhl import Stuhl
from tisch import Tisch
from sessel import Sessel
from ovalerTisch import OvalerTisch
from badewanne import Badewanne
from schrank import Schrank

from leinwand import *

### ---------------------------------------------------
class RaumplanerModell(object):
    '''Ist als Singleton modelliert'''

    __raumplanerModell = None

    @staticmethod
    def gibRaumplanerModell():
        if RaumplanerModell.__raumplanerModell == None:
            RaumplanerModell.__raumplanerModell=RaumplanerModell()
        return RaumplanerModell.__raumplanerModell
    
    def __init__(self):
        """Konstruktor"""
        ## Fuer Singleton: ----------------------------------------
        if RaumplanerModell.__raumplanerModell != None:
            raise Exception('Konstruktor nicht direkt aufrufen')
        ## --------------------------------------------------------
        self.__alleMoebel = []
        self.__ausgewaehlt = -1

    #### ----- Erzeugen -----
    def neu(self, klasse, *par, **kwargs):
        try:
            if klasse.__class__==str:
                klasse=eval(klasse)
            neu=klasse(*par, **kwargs)
            self.__alleMoebel.append(neu)
            neu.zeige()
            return neu
        except NameError as e:
            print(str(e))
            return None

    # Loeschen s.u.

    #### ----- sichern und laden -----
    def speichere(self, dateiname):
        '''Die Modell - Daten werden in eine Datei gespeichert.'''
        daten=[]
        for moebel in self.__alleMoebel:
            daten.append(moebel.gibDaten())
        try:
            f=open(dateiname,'w')
        except IOError as e:
            print('Datei existiert nicht, keine Daten geschrieben')
            return None
        f.write( str(daten) )
        f.close()
        return 'OK'

    def lade(self, dateiname):
        '''laedt die Daten aus der Datei'''
        try:
            f=open(dateiname,'r')
        except IOError as e:
            print('Datei existiert nicht, keine Daten gelesen')
            return None
        modellDatenString=f.read()
        f.close()
        self.loescheAktuellesModell()
        modellDaten=eval(modellDatenString)
        self.erzeugeModellAusDaten(modellDaten)
        return 'OK'

    def loescheAktuellesModell(self):
        for moebel in self.__alleMoebel:
            moebel.Verberge()
            del(moebel)
        self.__ausgewaehlt = -1
    
    def erzeugeModellAusDaten(self, modellDaten):
        for daten in modellDaten:
            self.neu(daten[0], # Name der KLasse
                     daten[1], # x
                     daten[2], # y
                     daten[3], # b
                     daten[4], # t
                     daten[5], # w
                     daten[6], # randfarbe
                     daten[7]) # (fuell)farbe
    
    def bildExport(self):
        """Bild wird in Bild-Datei geschrieben"""
        Leinwand().bildExport()            

    #### ----- Auswahl -----
    def waehle(self,index=None):
        '''wählt das Moebel indexbasiert aus;
        None zum Weitersetzen der Auswahl, -1 zum Beenden der Auswahl,
        nach dem letzten wird die Auswahl ebenfalls beendet'''
        if len(self.__alleMoebel)==0: # nichts auszuwaehlen
            return
        if index==-1: # Beenden der Auswahl
            aktuell = self.__ausgewaehlt
            self.__ausgewaehlt = -1
            if aktuell!=-1: # Darstellung aktualisieren
                self.__alleMoebel[aktuell].verberge()
                self.__alleMoebel[aktuell].waehle(False)
                self.__alleMoebel[aktuell].zeige()
            return
        if index!=None: # Indexfehler pruefen
            if index<-1: return
            if index>=len(self.__alleMoebel): return
            ## sonst auswaehlen und markieren
            aktuell = self.__ausgewaehlt
            self.__ausgewaehlt = index
            if aktuell!=-1: # Auswahl loeschen
                self.__alleMoebel[aktuell].verberge()
                self.__alleMoebel[aktuell].waehle(False)
                self.__alleMoebel[aktuell].zeige()
            self.__alleMoebel[self.__ausgewaehlt].verberge()
            self.__alleMoebel[self.__ausgewaehlt].waehle(True)
            self.__alleMoebel[self.__ausgewaehlt].zeige()
        if index==None: # Weitersetzen
            if self.__ausgewaehlt!=-1:
                self.__alleMoebel[self.__ausgewaehlt].verberge()
                self.__alleMoebel[self.__ausgewaehlt].waehle(False)
                self.__alleMoebel[self.__ausgewaehlt].zeige()
            self.__ausgewaehlt+=1
            if self.__ausgewaehlt==len(self.__alleMoebel):
                self.__ausgewaehlt=-1
                return
            self.__alleMoebel[self.__ausgewaehlt].verberge()
            self.__alleMoebel[self.__ausgewaehlt].waehle(True)
            self.__alleMoebel[self.__ausgewaehlt].zeige()                

    def gibAusgewaehltes(self):
        if self.__ausgewaehlt<0: return None
        else: return self.__alleMoebel[self.__ausgewaehlt]

    def loescheAusgewaehltes(self):
        if self.__ausgewaehlt>=0:
            self.gibAusgewaehltes().verberge()
            self.__alleMoebel.remove(self.gibAusgewaehltes())
            self.__ausgewaehlt=-1

    #### ----- Aktionen -----
    def horizontalBewegen(self, weite):
        """Bewegt das Ausgewaehlte horizontal
        reicht die Anforderung ggf weiter"""
        if self.__ausgewaehlt>=0:
            gewaehlt=self.gibAusgewaehltes()
            gewaehlt.horizontalBewegen(weite)
        
    def vertikalBewegen(self, weite):
        """Bewegt das Ausgewaehlte vertikal
        reicht die Anforderung ggf weiter"""
        if self.__ausgewaehlt>=0:
            gewaehlt=self.gibAusgewaehltes()
            gewaehlt.vertikalBewegen(weite)
        
    def drehe(self, winkel):
        """Dreht das Ausgewaehlte
        reicht die Anforderung ggf weiter"""
        if self.__ausgewaehlt>=0:
            gewaehlt=self.gibAusgewaehltes()
            gewaehlt.drehe(winkel)
        
    def farbeAendern(self, neueFarbe):
        """Dreht das Ausgewaehlte
        reicht die Anforderung ggf weiter"""
        if self.__ausgewaehlt>=0:
            gewaehlt=self.gibAusgewaehltes()
            gewaehlt.farbeAendern(neueFarbe)

    def randfarbeAendern(self, neueFarbe):
        """Dreht das Ausgewaehlte
        reicht die Anforderung ggf weiter"""
        if self.__ausgewaehlt>=0:
            gewaehlt=self.gibAusgewaehltes()
            gewaehlt.randfarbeAendern(neueFarbe)

    def gibAlleMoebel(self):
        return self.__alleMoebel

##### ---------------------------------------------------
##import wx
##from grafikfenster import *
##class MoebelApp(wx.App):
##    def OnInit(self):
##        self.fenster = GrafikFenster(None, "Raumplaner-Grafik")
##        self.SetTopWindow(self.fenster)
##        self.fenster.Show(True)
##        self.fenster.panel.Refresh()
##        self.fenster.ZeigeShellFrame()
##        #self.fenster.ZeigeFillingFrame()
##        self.modell=RaumplanerModell.GibRaumplanerModell()
##        ##self.TestAnwendung()
##        return True
##
##    def TestAnwendung(self):
##        global modell, schrank, sessel, stuhl, tisch
##        modell=self.modell
##        schrank=self.modell.Neu(Schrank,280, 40, 100, 60, 90, 'black', True)
##        stuhl=self.modell.Neu(Stuhl,10, 30, 40, 40, 270, 'blue', True)
##        tisch=self.modell.Neu(Tisch,60, 20, 120, 60, 0, 'red', True)
##        sessel=self.modell.Neu(Sessel, 100, 130, 60, 60, 0, 'orange', True)
##        sofa=self.modell.Neu(Sofa, 70, 290, 120, 60, 180, 'orange', True)
##        ovaler=self.modell.Neu(OvalerTisch, 70, 200, 120, 80, 0, 'brown', True)
##
##### ---------------------------------------------------
##if __name__ == '__main__':
##    app = MoebelApp(redirect=False)
##    # Parameterwert True, wenn Ausgaben in der Standard E/A angezeigt werden sollen
##    app.MainLoop()
